#!/bin/bash
#
# LICENSED MATERIALS - PROPERTY OF IBM
#
# 5725-C25
#
# (C) Copyright IBM Corp. 2012
#
# ALL RIGHTS RESERVED
#
# US Government Users Restricted Rights -c
# Use, duplication or disclosure restricted by
# GSA ADP Schedule Contract with IBM Corp.
#

MigratePEMCerts()
{
    echo "Begin ..."
    echo "Migrate PEM Certificates to the Working Keystore"
    echo "------------------------------------------------"

    LEAFCERTS=$1

    while ! [ -d "${LEAFCERTS}" ]; do
        echo "Specify leaf-certs directory containing PEM certificates (e.g. ../leafcerts):"
        read LEAFCERTS
    done

    if [ -e "${KDBPATH}" ]; then
        if ! [ -w "${KDBPATH}" ]; then 
            echo "${KDBPATH} exists but you do not have write access to it." 1>&2
            exit 1
        fi

        if ! [ -r "${KDBPATH}" ]; then 
            echo "${KDBPATH} exists but you do not have read access to it." 1>&2
            exit 1
        fi

        echo "${KDBPATH} already exists - adding leaf certs to this certificate store."
    else
        echo "${KDBPATH} does not already exist - a new keystore will be created."

        cmd="gsk8capicmd -keydb -create -db '${KDBPATH}' -pw '${PASSWD}' -type cms -expire 7300 -stash -f -fips true"
        echo "${cmd}"
        if ! eval "${cmd}" || ! [ -w "${KDBPATH}" ]; then
            echo "Failed to create keystore" 1>&2
            exit 1
        else
            chmod a+rw ${KDBPATH} ${KDBPATH%.*}.sth
        fi
    fi

    echo "Migrating PEM certificates to the keystore."

    (cd "${LEAFCERTS}"; ls -1 *.pem) | while read pem; do
        label="${pem%.pem}"
        cmd="gsk8capicmd -cert -add -db '${KDBPATH}' -type cms -stashed -label '${label}' -file '${LEAFCERTS}/${pem}' -format ascii -fips true"
        echo "${cmd}"
        eval "${cmd}"
        rc=$?
        case ${rc} in
            23) echo "Cerfificate ${pem} already exists in keystore. Ignoring." ;;
            0)  ;;
            *)  echo "Unable to add ${pem} to certificate store (exit status ${rc})." 1>&2
                result=1
                ;;
        esac
    done
}

AddPKCS12()
{
    local pkcs12file=$1
    local label=$(basename $(echo ${pkcs12file%.*}))

    cmd="gsk8capicmd -cert -import -db '${pkcs12file}' -pw '${PASSWD}' -label '${label}' -target '${KDBPATH}' -target_pw '${PASSWD}' -fips true"
    eval "${cmd}"
    result=$?
}

AddCert()
{
    local certfile=$1
    local label=$(basename $(echo ${certfile%.*}))

    cmd="gsk8capicmd -cert -add -db '${KDBPATH}' -type cms -stashed -label '${label}' -file '${certfile}' -format ascii -fips true"
    eval "${cmd}"
    result=$?
}

RemoveCert()
{
    local label=$1

    cmd="gsk8capicmd -cert -delete -label '${label}' -db '${KDBPATH}' -pw '${PASSWD}' -fips true"
    eval "${cmd}"
    result=$?
}

List()
{
    cmd="gsk8capicmd -cert -list -db '${KDBPATH}' -pw '${PASSWD}' -fips true"
    eval "${cmd}"
    result=$?
}

GSK8BIN="/usr/lib/gsk8/bin"

while ! [ -x "${GSK8BIN}/gsk8capicmd" ]; do
    echo "Specify the location of gsk8capicmd (e.g. /opt/IBM/product/gsk8/bin):"
    read GSK8BIN
done

GSK8LIB="${GSK8BIN%/*}/lib"

case "$(uname -s)" in
    Linux|SunOS|*)
	SHLIBSUFF=".so"
	LIBPATH_VAR=LD_LIBRARY_PATH
	;;
    AIX)
	SHLIBSUFF=".so"
	LIBPATH_VAR=LIBPATH
	;;
    HP-UX)
	SHLIBSUFF=".sl"
	LIBPATH_VAR=SHLIBPATH
	;;
esac

while ! [ -r "${GSK8LIB}/libgsk8km${SHLIBSUFF}" ]; do
    echo "Specify the location of gsk8 library files (e.g. /opt/IBM/product/gsk8/bin):"
    read GSK8LIB
done


PASSWD=ibmss01
KDB=iss-working.kdb
KDBPATH=$2

PATH="${PATH:+${PATH}:}${GSK8BIN}"
eval "${LIBPATH_VAR}=\${${LIBPATH_VAR}:+\${${LIBPATH_VAR}}:}${GSK8LIB}"

export PATH ${LIBPATH_VAR}

result=0
case $1 in
    -m)
        MigratePEMCerts $3
        ;;
    -p)
        AddPKCS12 $3
        ;;
    -a)
        AddCert $3
        ;;
    -r)
        RemoveCert $3
        ;;
    -l)
        List
        ;;
    *)
        result=1
        ;;
esac

exit ${result}
