#!/bin/bash
set -o nounset
#  This script is invoked by CRM via the popen() call and subscribes to CPE (Common Policy Editor) 
#  semantics.  This means that this script communicates results via stdout.  On Success, it should
#  echo "OK ".  Anything else will be considered a failure and reported back to SiteProtector.
#  So every error must return a string.
#
#  When changing this script, be careful when using commands that may generate output. 
#  (for example "cd -".  Be sure these commands redirect their output to /dev/null, or you'll
#  create a subtle bug for yourself to find.
#
outFile="/tmp/sensorPolicy.xml"
errFile="/tmp/setschema.log"
schemasJarPath="/var/www/lmi/public/schemas"
schemajar="schemas5_0.jar"

nameSpace=""
schemaVersion=""
spi=""
DB_CONF="/etc/iss/iss-dbd.conf"


while getopts "s:n:p:" opt; do
  case $opt in
    s ) schemaVersion=$OPTARG ;;
    n ) nameSpace=$OPTARG ;;
    p ) spi=$OPTARG ;;
  esac
done

if [ -f $errFile ]; then
  rm -f $errFile > /dev/null
fi

if [ "$nameSpace" = "" ] || [ "$schemaVersion" = "" ] || [ "$spi" = "" ]; then
	echo "Invalid Usage..  usage $0 -s schemaVersion -n nameSpace -p sensorPolicyInfo_file" 
	exit 1
fi


if [ -f $outFile ]; then
  echo "removing $outFile" >> $errFile
  rm -f $outFile > /dev/null
fi

if [ -f $spi ]; then
	cp -f $spi $spi.001 > /dev/null
	echo "Copying $spi to $spi.001" >> $errFile
fi

# Get the current schema info before we update it
Xalan $spi /etc/crm/transforms/xslt/SensorPolicyInfo_1.1_List.xslt > /tmp/currentSchemas.lst 2>/dev/null
oldSchemaVersion=`cat /tmp/currentSchemas.lst | grep $nameSpace | awk -F , '{print $6}'`
oldSchemaPath=`cat /tmp/currentSchemas.lst | grep $nameSpace | awk -F , '{print $2}'`
oldSchemaName=`cat /tmp/currentSchemas.lst | grep $nameSpace | awk -F , '{print $3}'`
rm -f /tmp/currentSchemas.lst >/dev/null 2>&1
oldVersionStr=`echo ${oldSchemaVersion} |  awk -F . '{print $1 "_" $2 "_" $3}'`
newVersionStr=`echo ${schemaVersion} |  awk -F . '{print $1 "_" $2 "_" $3}'`
schemaBasename=`basename ${oldSchemaName} ${oldVersionStr}.xsd`
echo "oldVersionStr=${oldVersionStr} newVersionStr=${newVersionStr}" >> $errFile
echo "oldSchemaPath=${oldSchemaPath} schemaBasename=${schemaBasename}" >> $errFile

#  Run the transform
Xalan -o $outFile -p paramSchemaVersion "'$schemaVersion'" -p paramNameSpace "'$nameSpace'" $spi /etc/crm/transforms/xslt/SensorPolicyInfo_1.2_Mod_Compatible.xslt 2>> $errFile
result=$?
if [ $result -eq 0 ]; then
   mv -f $outFile $spi > /dev/null
else
   echo "Transforming $spi has failed.  Xalan returned $result" >> $errFile
   #  send something back to crm
   echo "Xalan failed to transform  $nameSpace to $schemaVersion"
   exit $result
fi

# Test the result.  Ensure that the file can be reread and that the version
# attribute did change.
#
changedVersion=`Xalan $spi /etc/crm/transforms/xslt/SensorPolicyInfo_1.1_List.xslt | grep $nameSpace | awk -F , '{print $6}'`
result=$?
if [ $result -ne 0 ]; then
  cp -f $spi.001 $spi > /dev/null
  echo "ERROR:  Test of transform file failed." >> $errFile
  echo "Validation of transformed SensorPolicyInfo failed"
  exit 1
fi

if [ "$changedVersion" != "$schemaVersion" ]; then
  cp -f $spi.001 $spi > /dev/null
  echo "ERROR:  Test of transform file failed.  Versions don't match. $changedVersion was read from $spi" >> $errFile
  #  This line for the benefit of CRM
  echo "ERROR:  Test of transform file failed.  Versions don't match. $changedVersion was read from $spi" 
  exit 1
else
  echo "Schema version succesfully changed to $changedVersion for $nameSpace" >> $errFile
fi
result=1
# rebuild the schemas.jar file
if [ -f $schemasJarPath/$schemajar ]; then
	echo "Building the jar." >> $errFile
	cp -f $schemasJarPath/$schemajar $schemasJarPath/$schemajar.001 > /dev/null

	# Copy the .properties files for previous schema version to the new version
	# This is a temporary half fix until SP starts sending the properties files with the schema update
	rm -rf /tmp/schemaupdate >/dev/null 2>&1
	mkdir /tmp/schemaupdate >/dev/null 2>&1
	pushd /tmp/schemaupdate >/dev/null 2>&1 
	/usr/bin/unzip $schemasJarPath/$schemajar >/dev/null 2>&1
	if [ -f "${oldSchemaPath}/${schemaBasename}${oldVersionStr}.properties" -a ! -f "${oldSchemaPath}/${schemaBasename}${newVersionStr}.properties" ] ; then
		cp  -f "${oldSchemaPath}/${schemaBasename}${oldVersionStr}.properties" "/etc/crm/schemas/${oldSchemaPath}/${schemaBasename}${newVersionStr}.properties" >/dev/null 2>&1
	    echo "Copied properties file" >> $errFile
	fi
	if [ -f "${oldSchemaPath}/${schemaBasename}${oldVersionStr}_helpmap.properties" -a ! -f "${oldSchemaPath}/${schemaBasename}${newVersionStr}_helpmap.properties" ] ; then
		cp  -f "${oldSchemaPath}/${schemaBasename}${oldVersionStr}_helpmap.properties" "/etc/crm/schemas/${oldSchemaPath}/${schemaBasename}${newVersionStr}_helpmap.properties" >/dev/null 2>&1
	    echo "Copied helpmap properties file" >> $errFile
	fi
	popd >/dev/null 2>&1
	rm -rf /tmp/schemaupdate >/dev/null 2>&1

	cd /etc/crm/schemas > /dev/null
	/usr/bin/zip -qr $schemasJarPath/$schemajar cml Iss1_0_0.xsd Iss1_0_1.xsd > /dev/null
	result=$?
	echo "zip returned $result" >> $errFile
	
	# remove the files we created for the temporary fix
	rm -f "/etc/crm/schemas/${oldSchemaPath}/${schemaBasename}${newVersionStr}.properties" >/dev/null 2>&1
	rm -f "/etc/crm/schemas/${oldSchemaPath}/${schemaBasename}${newVersionStr}_helpmap.properties" >/dev/null 2>&1

	cd - > /dev/null
	if [ $result -ne 0 ]; then
		echo "Error building jar '$schemajar'" >> $errFile
		cp -f $schemasJarPath/$schemajar.001 $schemasJarPath/$schemajar > /dev/null
		rm  -f $schemasJarPath/$schemajar.001 > /dev/null
		echo "Returning $result" >> $errFile
		echo "Failed to create new Schemas.jar file"
		exit $result
	else
	  echo "OK "
	  exit 0 
	fi
else
	echo "Could not locate the schemas jar file.  SetSchemaRequest has failed"
	exit 1
fi



