#!/bin/bash 
set -o nounset
#
#  Wrapper script to push a policy to the sensor.  Call this with one paramter
#  the full path to the policy name.
#

#  These are the codes returned to the caller.  These, in conjunction with std::out 
#  should be used to determine success.

declare -i ReturnCode_FileManuallyCopied=0
declare -i ReturnCode_FileCopyFailed=2
declare -i ReturnCode_InvalidUsage=3
declare -i ReturnCode_SourceNotFound=4
declare -i ReturnCode_InvalidSourceFormat=5
declare -i ReturnCode_InvalidNamespace=6
declare -i ReturnCode_NoHandler=7
declare -i ReturnCode_PPDNotRunning=8
declare -i ReturnCode_NoHandlerFile=9
declare -i ReturnCode_SensorNotRunning=10

POLICYINFOXFORM="/etc/crm/transforms/xslt/PolicyNamespaceAndVersion.xslt"
SPIXFORM="/etc/crm/transforms/xslt/SensorPolicyInfo_1.2_List.xslt"
SPIFile="/etc/crm/SensorPolicyInfoG.xml"

# ManuallyPlaceFile is designed to be invoked only when it's valid to bypass CRM to install files
# Specifically, this would be during setup, prior to when CRM is expected to be running.  Pass
# in the input file name, Location to place the file into, and the Namespace.
function ManuallyPlaceFile()
{
  #  Manually place file into directory.  Making backup of original file
  #  first....
  PolicyDestDir="/etc/crm/policies/$2"
  DestFileName=`Xalan $SPIFile $SPIXFORM | grep $3 |  grep $Version |awk -F, '{ print $4'}` > /dev/null 
  install -m 644 -o root -g root  $1 $PolicyDestDir/$DestFileName >&2
  if [ $? -eq 0 ]; then
    echo "OK  "
    exit $ReturnCode_FileManuallyCopied
  else
    exit $ReturnCode_FileCopyFailed
  fi
}

if [ $# -eq 0 ]; then
  echo "Usage:  $0 Full_Path_To_Policy"
  exit $ReturnCode_InvalidUsage
fi

PolicyFile=$1

if [ ! -f $PolicyFile ]; then
  echo "ERROR:  PolicyFile $PolicyFile does not exist.  Exiting."  >&2
  exit $ReturnCode_SourceNotFound
fi

#  Get the Namespace and Version from the policy itself..
NSVer=`Xalan $PolicyFile $POLICYINFOXFORM 2> /dev/null`
if [ $? -ne 0 ]; then 
  # If this is invalid (or not even) xml.  Bark out the error message, then quit.
  echo "ERROR: reading $PolicyFile to determine the namespace and version.  Is this policy valid?" >&2
  exit $ReturnCode_InvalidSourceFormat
fi
NS=`echo $NSVer | awk -F, '{print $1}'`
#  Look for a valid namespace.  Generally begins with http://www
echo $NS | egrep -e http://www > /dev/null
if [ $? -ne 0 ]; then
  echo "ERROR:  Policy $PolicyFile does not appear to have a valid namespace." >&2
  exit $ReturnCode_InvalidNamespace
fi
Version=`echo $NSVer | awk -F, '{print $2}'`
#  From SensorPolicyInfo, get the handler
Handler=`Xalan $SPIFile $SPIXFORM | grep $NS |  grep $Version |awk -F, '{ print $9'}` > /dev/null 
#
if [ "$Handler" = "" ]; then
  echo "ERROR: No handler for Namespace $NS and Version $Version specified in $SPIFile.  Exiting." >&2
  exit $ReturnCode_NoHandler
fi


declare -i PPDRunning=0
declare -i DaemonRunning=0

#Get the Pid of the issDaemon.  Only 1 is running, so it's safe to use pidof.
pidof issDaemon > /dev/null
if [ $? -eq 0 ]; then
  DaemonRunning=1
fi

#To get the pid of the ppd, use the lock file.
declare PPDPID
if [ -f /var/run/issppd.pid ]; then
  pidof issppd > /dev/null
  if [ $? -eq 0 ]; then
    PPDRunning=1
  fi
fi

#  If the handler isn't running, then fail it.
if [ "$Handler" = "CRM" ]; then
  if [ $DaemonRunning -eq 0 ]; then
     if [ -f /etc/appliance/.confdone ]; then
       echo "ERROR:  Cannot push policy for Namespace $NS.  The sensor is not running" >&2
       exit $ReturnCode_SensorNotRunning
     else
       Location=`Xalan $SPIFile $SPIXFORM | grep $NS | awk -F, '{print $2}'` > /dev/null
       ManuallyPlaceFile $PolicyFile $Location $NS
     fi
  fi 
else
  if [ $PPDRunning -eq 0 ]; then
    echo "ERROR: Cannot push policy. Policy Processing Daemon is not running" >&2
    exit $ReturnCode_PPDNotRunning
  fi
  #  If the handler isn't CRM, then it must be a .so.  If that .so doesn't exist, then
  #  fail the policy push as well
  if [ ! -f "$Handler" ]; then
    echo "ERROR: Cannot push policy.  Handler $Handler for Namespace $NS, version $Version does not exist." >&2
    exit $ReturnCode_NoHandlerFile 
  fi
fi

#
#  Regardless of the handler,  the policy will get pushed through CRM since it knows how to forward.  The idea
#  is that whenever possible, let CRM have the opportunity to examine the policy.
#
if [ $DaemonRunning -eq 1 ]; then
  issasApache -a apf -pns $NS -sv $Version -pfn $PolicyFile
  exit $?
else
  #push through ppd
  /usr/bin/issppdcom -f $PolicyFile
  exit $?
fi
