class LmiFilter
  require 'enumerator'

  attr_accessor :search_strings, :params, :filter_type

  def initialize(params={}, stored=false)
    Utility.recursive_symbolize_keys!(params)
    @params = params

    if stored
      obj = FilterTag.find(params[:id])
      @search_strings = parse_stored_filter(obj)
    else
      @search_strings = parse_filter(@params)
    end
  end

  def filter_type
    sanitize(@params[:type])
  end

  def filter_tag
	htmlSanitize(@params[:filter_tag])
  end

  def self.destroy(id)
    FilterTag.destroy(id)
  end

  #Methods to parse filters stored in the database d
  def parse_stored_filter(filter)
    if(filter_type == "alert")
      return parse_stored_alert_filter(filter)
    else
      return parse_stored_filestore_filter(filter)
    end
  end

  def parse_filter(arg)
    if(filter_type == "alert")
      return parse_alert_filter(arg)
    else
      return parse_filestore_filter(arg)
    end
  end

  #Methods to parse ad hoc filters
  def parse_alert_filter(arg)
    return {} if arg.blank?
    fhash = {}
    fhash[:filter] = arg[:filter].map{ |k, v| {:text=>v[:text], :coltype=>v[:coltype]} } unless arg[:filter].blank?
    fhash[:date] = arg[:date] unless arg[:date].blank? if arg[:time_select] == "custom"
    fhash[:filter_type] = arg[:filter_type]
    fhash[:time_select] = arg[:time_select]
    fhash
  end

  def parse_filestore_filter(arg)
    return {} if arg.blank?
    fhash = {}
    fhash[:filter] = arg[:filter].map{ |k, v| v } unless arg[:filter].blank?
    fhash[:date] = arg[:date] unless arg[:date].blank? if arg[:time_select] == "custom"
    fhash[:filter_type] = arg[:filter_type]
    fhash[:time_select] = arg[:time_select]
    fhash
  end

  #Methods to parse filters stored in the database
  def parse_stored_alert_filter(record)
    tmphash = {}
    fhash = {}
    record.filter_criterias.each_with_index{ |c, i| tmphash.merge!({"criteria_#{i+1}.to_sym"=>{:text=>c.criteria, :coltype=>c.coltype}}) }
    fhash[:filter] = tmphash
    fhash[:date] = {:start_time => record.start_time, :end_time => record.end_time} if record.meta_time == "custom"
    fhash[:time_select] = record.meta_time
    fhash[:filter_type] = record.filter_type
    parse_alert_filter(fhash)
  end

  def parse_stored_filestore_filter(record)
    tmphash = {}
    fhash = {}
    record.filter_criterias.each_with_index{ |c, i| tmphash.merge!({"textbox_#{i}"=>c.criteria}) }
    fhash[:filter] = tmphash
    start_time = record.start_time.blank? ? nil : record.start_time
    end_time = record.end_time.blank? ? nil : record.end_time
    fhash[:date] = {:start_time => record.start_time, :end_time => record.end_time} unless record.start_time.blank? if record.meta_time == "custom"
    fhash[:time_select] = record.meta_time
    fhash[:filter_type] = record.filter_type
    parse_filestore_filter(fhash)
  end

  #Methods to build filters from filter database records
  def build_alert_filter(arg)
    record.filter_criterias.map{ |c| c.criteria }
  end

  ################################ Log Filter Save ##################################
  ################################ Log Filter Save ##################################
  ################################ Log Filter Save ##################################
  def save
    tag = FilterTag.new
    tag[:tag] = filter_tag()
    tag[:filter_type] = filter_type()
    tag[:query_string] = build_query_string()
    tag[:meta_time] = meta_time()
    tag[:end_time] = end_time()
    tag[:start_time] = start_time()

    begin
      tag.save!
      search_strings[:filter].each do |v|
        args = (filter_type == "alert" ? {:criteria => htmlSanitize(v[:text]), :coltype => sanitize(v[:coltype])} : {:criteria => htmlSanitize(v)})
        args.merge!(:filter_tag_id => tag.id)
        FilterCriteria.new(args).save!
      end
    rescue ActiveRecord::RecordInvalid
      # TODO we should allow the user to update a tag.  perhaps offer a confirmation msg.
    end
  end

  def start_time
    date = ""
    if(!search_strings[:date].blank?)
      sdate = search_strings[:date][:start_time]
      date = sdate.blank? ? "" : sdate
    elsif(meta_time?)
      date = calculate_start_time()
    end
    sanitize(date)
  end

  def end_time
    if(search_strings.has_key?(:date))
      edate = search_strings[:date][:end_time]
      edate.blank? ? "" : sanitize(edate)
    end
  end

  def meta_time
    sanitize(@search_strings[:time_select])
  end

  def meta_time?
    (meta_time == "day" || meta_time == "hour" || meta_time == "week" || meta_time == "month")
  end

  def calculate_start_time
    ret = ""
    case meta_time
    when "hour" then ret = previous_hour
    when "day" then ret = previous_day
    when "week" then ret = previous_week
    when "month" then ret = previous_month
    end
    return (ret * 1000).to_s
  end

  def previous_hour
    Time.now.to_i - 3600
  end

  def previous_day
    Time.now.to_i - 86400
  end

  def previous_week
    Time.now.to_i - 604800
  end

  def previous_month
    Time.now.to_i - 2629744
  end

  def has_date_filter?
    search_strings.has_key?(:date) || meta_time?
  end

  def has_line_filter?
    search_strings.has_key?(:filter)
  end

  def empty?
    # Remove if the new check doesn't produce problems.
    #search_strings.empty? && !(meta_time? || @params.has_key?(:date) || @params.has_key?(:filter))
     !(meta_time? || @params.has_key?(:date) || @params.has_key?(:filter) || (@params.has_key?(:action) && @params[:action] == "load_saved_filter"))
  end

  def self.blank
    new({})
  end

  def build_query_string
    query_string = ""
    fstrings = []

    unless @params[:filter].blank?
      if filter_type == "alert"
        fstrings = search_strings[:filter].enum_with_index.map do |val, index|
          val[:coltype] = sanitize(val[:coltype])
          val[:text] = htmlSanitize(val[:text])
          "filter[criteria_#{index+1}][text]=#{val[:text]}&filter[criteria_#{index+1}][coltype]=#{val[:coltype]}"
        end
      else

        fstrings = @params[:filter].enum_with_index.map{|val, index| "filter[textbox_#{index+1}]=#{htmlSanitize(val[1])}"}
      end
    end

    fstrings.concat(add_dates) unless @params[:date].blank? && !meta_time?
    fstrings.join("&")
  end

  def add_dates
    return [] if @params[:date].blank? && @params[:time_select].blank?
    dates = []
    unless @params[:date].blank?
      dates = @params[:date].map{|k, v| "date[#{k}]=#{sanitize(v)}"}
    end
    dates << "time_select=#{sanitize(@params[:time_select])}"
    dates
  end

  def do_filter(results)
    results if empty?
    if has_date_filter?
      results = do_time_filter(results)
    end

    if has_line_filter?
      results = do_line_filter(results)
    end

    results
  end

  def convert_date_to_int(date)
    parts = Array.new
    parts = date.split(/\s+/,4)
    logtime = parts[2].split(/:/)
    Time.local(parts[3],parts[0].downcase,parts[1],logtime[0],logtime[1]).to_i
  end

  def determine_time_filter_values
    time_start = convert_date_to_int("Jan 1 00:00:00 1980")
    time_end = convert_date_to_int("Dec 31 23:59:00 2037")

    if ! start_time.nil?  then
      time_start = start_time.to_i/1000
    end
    if meta_time?
      time_end = Time.now.to_i
    elsif ! end_time.nil? then
      time_end = end_time.to_i/1000
    end

    return time_start, time_end
  end


  def do_time_filter(readlines)
    results = Array.new

    start_int, end_int = determine_time_filter_values

    file_start = convert_date_to_int(readlines[0])
    file_end = convert_date_to_int(readlines[-1])

    if start_int > file_end || end_int < file_start then
      return results
    elsif file_start >= start_int && file_end <= end_int then
      results = readlines
    else
      readlines.each do |l|
        date_int = convert_date_to_int(l)
        if date_int >= start_int && date_int <= end_int then
          results.push(l)
        end
      end
    end

    results
  end

  def do_line_filter(results)
    search_strings[:filter].each{|f| results = results.grep(/#{sanitize(f)}/i) }
    results
  end

  def sanitize(str)
    return "" if str.blank?
    str = str.gsub(/\\|\/|'|"|\||\&|\$|\%|\@|\;|\(|\)/, '') unless NetAddressValidator.validate(str)
    str = CGI.escapeHTML(str)
    str
  end
  
  def htmlSanitize(str)
    return "" if str.blank?
    str = CGI.escapeHTML(str) unless NetAddressValidator.validate(str)
    str
  end
end
