class ManagementInterface < ActiveRecord::Base
  require 'rexml/document'
  include REXML
  attr_accessor :search_paths
  #General network information
  validates_presence_of :host_name #, :sensor_name
  validates_presence_of :primary_dns, :if => :static_dns?, :message => "can't be blank if Manual DNS option is selected"
  validates_inclusion_of :Manual_DNS, :in => [true], :if => :static_ipv4_interface?, :message => "is required if Manual IPv4 is selected"  
  validates_short_hostname :host_name, :if=> :short_hostname?
  validates_hostname :host_name, :if=> :fqdn_hostname?
  validates_length_of :host_name, :maximum => 63, :if=>:fqdn_hostname?, :message => "is too long (maximum 50 characters or 63 characters using a FQDN)"
  validates_length_of :host_name, :maximum => 50, :if=>:short_hostname?, :message => "is too long (maximum 50 characters or 63 characters using a FQDN)"
  validates_ip_address :primary_dns, :secondary_dns, :tertiary_dns, :allow_nil => true

  #IPv4 validations
  validates_presence_of :ipv4_netmask, :if => :static_ipv4_interface?
  validates_presence_of :ipv4_gateway, :if => :static_ipv4_interface?
  validates_ipv4 :ipv4_address, :ipv4_gateway, :ipv4_netmask, :if => :static_ipv4_interface?


  #IPv6 validations
  validates_presence_of :ipv6_prefix_length, :if => :static_ipv6_interface?
  validates_presence_of :ipv6_gateway, :if => :static_ipv6_interface?
  validates_ipv6 :ipv6_address, :if => :static_ipv6_interface?
  validates_ipv6 :ipv6_gateway, :if => :static_ipv6_interface?
  validates_numericality_of :ipv6_prefix_length, :integer_only => true, :greater_than_or_equal_to => 0, :less_than_or_equal_to => 64, :message => "must be a decimal number between 1 and 128", :if => :static_ipv6_interface?
  #validates_numericality_of :ipv6_prefix_length, :integer_only => true, :if => :static_ipv6_interface?
  #validates_inclusion_of :ipv6_prefix_length, :in => 1..128, :message => "must be a decimal number between 1 and 128"

  def before_save
    if !static_dns?
      self.primary_dns = nil
      self.secondary_dns = nil
      self.tertiary_dns = nil
      self.dns_search_path = nil
    end
  end 

  def parse_search_path
    self.dns_search_path.split(" ")
  end

  #private
  def static_interface?
    static_ipv4_interface? or static_ipv6_interface?
  end

  def static_ipv4_interface?
    self.ipv4_addressing_method == 2
  end
  
  def static_ipv6_interface?
    self.ipv6_addressing_method == 2
  end

  def search_path?
    !self.dns_search_path.blank?
  end

  def static_dns?
    self.is_static_dns
  end
  
  def Manual_DNS
    static_dns?
  end  

  def self.search_paths(search_path)
    paths = search_path.split(" ").join("|")
    paths
  end

  def short_hostname?
    (self.host_name =~ /\./).nil?
  end

  def fqdn_hostname?
    self.host_name =~ /\./
  end

  def self.configure
    mi = find(:first)
    args = {}    
    args["pHostName"] = mi.host_name
    args["pEnableMDns"] = (mi.multicast_disabled ? "false" : "true")
    args["pDnsIpType"] = (mi.is_static_dns ? "IPv4OrIPv6Address" : "AutoAssigned") 
    args["pDnsPrimary"] = (mi.is_static_dns ? mi.primary_dns : "")
    args["pDnsSecondary"] = (mi.is_static_dns ? mi.secondary_dns : "")
    args["pDnsTertiary"] = (mi.is_static_dns ? mi.tertiary_dns : "")
    args["pDnsList"] = mi.dns_search_path.blank? ? "" : search_paths(mi.dns_search_path)
    args["pMgmtSpeed"]  = "Auto"
    args["pMgmtDuplex"] = ""
    args["pMgmtIpv4Type"] = (mi.ipv4_addressing_method == 2 ? "Static" : "DHCP")
    args["pMgmtIpv4Addr"] = (mi.ipv4_addressing_method == 2 ? mi.ipv4_address : "")
    args["pMgmtIpv4Mask"] = (mi.ipv4_addressing_method == 2 ? mi.ipv4_netmask : "")
    args["pMgmtIpv4Gateway"] = (mi.ipv4_addressing_method == 2 ? mi.ipv4_gateway : "")
    args["pMgmtIpv6Type"] = (mi.ipv6_addressing_method == 2 ? "Static" : "Auto")
    args["pMgmtIpv6Addr"] = (mi.ipv6_addressing_method == 2 ? mi.ipv6_address : "")
    args["pMgmtIpv6Prefix"] = (mi.ipv6_addressing_method == 2 ? mi.ipv6_prefix_length : "")
    args["pMgmtIpv6Gateway"] = (mi.ipv6_addressing_method == 2 ? mi.ipv6_gateway : "")
    args["pKillSpeed"] = "Auto"
    args["pKillDuplex"] = ""

    Setup.configure_management_interface(args)
  end
  
  def self.load_from_policy
  #  nwsettings = {:host_name =>"", :sensor_name => "", :multicast_disabled => ""}
  #  nwsettings.merge({:ipv4_addressing_method => "", :ipv4_address => "", :ipv4_gateway => "", :ipv4_netmask => ""})
  #  nwsettings.merge({:ipv6_addressing_method => "", :ipv6_address => "", :ipv6_gateway => "", :ipv6_prefix_length => ""})
  #  nwsettings.merge({ :is_static_dns => "", :primary_dns => "", :secondary_dns => "", :tertiary_dns => "", :dns_search_path => ""})
  
    nwsettings = find(:first)
    
    if nwsettings.blank?
      nwsettings = ManagementInterface.new
    end
  
    policy = Document.new File.new("#{Setup.policy_transforms[:network][:policy]}")
    nwsettings[:sensor_name] = Appliance.sensor_name
    nwsettings[:host_name] = policy.elements['//Config/attribute::HostName'].value
    
    unless policy.elements['//Config/DNS/attribute::mDNSResponder'].blank?
      nwsettings[:multicast_disabled] = policy.elements['//Config/DNS/attribute::mDNSResponder'].value == "true" ? false : true
    end
    
    nwsettings[:is_static_dns] = policy.elements['//Config/DNS/AutoAssigned'].nil? ? true : false
    
    nwsettings[:ipv4_addressing_method] = policy.elements['//Config/MgmtInterface/IPv4/DHCP'].nil? ? 2 : 1
    nwsettings[:ipv6_addressing_method] = policy.elements['//Config/MgmtInterface/IPv6/Auto'].nil? ? 2 : 1
    
    if nwsettings[:is_static_dns]
      
      unless policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::PrimaryDNS'].blank?
        nwsettings[:primary_dns] = policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::PrimaryDNS'].value
      end
      
      unless policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::SecondaryDNS'].blank?
        nwsettings[:secondary_dns] = policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::SecondaryDNS'].value
      end
      
      unless policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::TertiaryDNS'].blank?
        nwsettings[:tertiary_dns] = policy.elements['//Config/DNS/IPv4OrIPv6Address/attribute::TertiaryDNS'].value
      end
      
      nwsettings[:dns_search_path] = XPath.match(policy, "//Config/DNS/DNSSearchPathList").map{|p| p.attributes["DomainName"]}.join(" ")
    end
    
    if nwsettings[:ipv4_addressing_method] == 2
      unless policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::IpAddr'].blank?
        nwsettings[:ipv4_address] = policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::IpAddr'].value
      end
      
      unless policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::SubnetMask'].blank?
        nwsettings[:ipv4_netmask] = policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::SubnetMask'].value
      end
      
      unless policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::GatewayIpAddr'].blank?
        nwsettings[:ipv4_gateway] = policy.elements['//Config/MgmtInterface/IPv4/Static/attribute::GatewayIpAddr'].value
      end
    end
    
    if nwsettings[:ipv6_addressing_method] == 2
      unless policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::IpAddr'].blank?
        nwsettings[:ipv6_address] = policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::IpAddr'].value
      end
      unless policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::Prefix'].blank?
        nwsettings[:ipv6_prefix_length] = policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::Prefix'].value
      end
      
      unless policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::GatewayIpAddr'].blank?
        nwsettings[:ipv6_gateway] = policy.elements['//Config/MgmtInterface/IPv6/Static/attribute::GatewayIpAddr'].value
      end
    end

    begin
      nwsettings.save!
    rescue
      RAILS_DEFAULT_LOGGER.error "ERROR WHILE ATTEMPTING TO IMPORT MANAGEMENT INTERFACE POLICIES"
    end
  end
end
